import discord
from v6d3music.core.caching import Caching
from v6d3music.core.queueaudio import QueueAudio
from v6d3music.utils.assert_admin import assert_admin

from ptvp35 import *
from v6d2ctx.context import Explicit


class MainAudio(discord.PCMVolumeTransformer):
    def __init__(self, db: DbConnection, queue: QueueAudio, volume: float):
        self.db = db
        self.queue = queue
        super().__init__(self.queue, volume=volume)

    async def set(self, volume: float, member: discord.Member):
        assert_admin(member)
        if volume < 0.01:
            raise Explicit('volume too small')
        if volume > 1:
            raise Explicit('volume too big')
        self.volume = volume
        await self.db.set(member.guild.id, volume)

    @classmethod
    async def create(cls, caching: Caching, db: DbConnection, queues: DbConnection, guild: discord.Guild) -> 'MainAudio':
        return cls(db, await QueueAudio.create(caching, queues, guild), volume=db.get(guild.id, 0.2))
